﻿using System;
using System.Drawing;
using System.IO;
using System.Windows;
using System.Windows.Forms;
using Palmmedia.AudioStarter.ViewModel;

namespace Palmmedia.AudioStarter
{
    /// <summary>
    /// Interaction logic for App.xaml
    /// </summary>
    public partial class App : System.Windows.Application
    {
        /// <summary>
        /// The notify/tray icon.
        /// </summary>
        private NotifyIcon notifyIcon;

        /// <summary>
        /// The <see cref="AudioService"/>.
        /// </summary>
        private AudioService audioService;

        /// <summary>
        /// Raises the <see cref="E:System.Windows.Application.Startup"/> event.
        /// </summary>
        /// <param name="e">A <see cref="T:System.Windows.StartupEventArgs"/> that contains the event data.</param>
        protected override void OnStartup(StartupEventArgs e)
        {
            base.OnStartup(e);

            this.ShutdownMode = System.Windows.ShutdownMode.OnExplicitShutdown;

            // Register handler for unhandled exceptions
            this.DispatcherUnhandledException += new System.Windows.Threading.DispatcherUnhandledExceptionEventHandler(this.App_DispatcherUnhandledException);

            this.CreateIcon();

            this.audioService = new AudioService();
        }

        /// <summary>
        /// Raises the <see cref="E:System.Windows.Application.Exit"/> event.
        /// </summary>
        /// <param name="e">An <see cref="T:System.Windows.ExitEventArgs"/> that contains the event data.</param>
        protected override void OnExit(ExitEventArgs e)
        {
            this.notifyIcon.Visible = false;
            base.OnExit(e);
        }

        /// <summary>
        /// Creates the icon.
        /// </summary>
        private void CreateIcon()
        {
            this.notifyIcon = new NotifyIcon();

            System.Reflection.Assembly a = System.Reflection.Assembly.GetExecutingAssembly();
            Stream stream = a.GetManifestResourceStream("Palmmedia.AudioStarter.microphone.ico");
            this.notifyIcon.Icon = new Icon(stream);
            this.notifyIcon.Text = "Audioservice";
            this.notifyIcon.Click += new EventHandler(this.NotifyIcon_Click);
            this.notifyIcon.Visible = true;

            ContextMenuStrip menu = new ContextMenuStrip();
            menu.Items.Add("Exit");
            menu.Items[0].Click += new EventHandler((s, e) => this.Shutdown());

            this.notifyIcon.ContextMenuStrip = menu;
        }

        /// <summary>
        /// Handles the Click event of the notifyIcon control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        private void NotifyIcon_Click(object sender, EventArgs e)
        {
            var mouseEvent = e as MouseEventArgs;

            if (mouseEvent != null && mouseEvent.Button == MouseButtons.Left)
            {
                var mainWindow = new MainWindow();
                mainWindow.DataContext = new MainWindowViewModel();
                mainWindow.Show();
            }
        }

        /// <summary>
        /// Handles the DispatcherUnhandledException event of the Application.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.Windows.Threading.DispatcherUnhandledExceptionEventArgs"/> instance containing the event data.</param>
        private void App_DispatcherUnhandledException(object sender, System.Windows.Threading.DispatcherUnhandledExceptionEventArgs e)
        {
            System.Windows.MessageBox.Show(e.Exception.Message);
            e.Handled = true;
        }
    }
}
