﻿using System.ComponentModel;
using Palmmedia.AudioStarter.Model;

namespace Palmmedia.AudioStarter.ViewModel
{
    /// <summary>
    /// ViewModel for 
    /// </summary>
    internal class RegisteredProgramViewModel : ViewModelBase, IDataErrorInfo
    {
        /// <summary>
        /// Initializes a new instance of the <see cref="RegisteredProgramViewModel"/> class.
        /// </summary>
        /// <param name="registeredProgram">The registered program.</param>
        public RegisteredProgramViewModel(RegisteredProgram registeredProgram)
        {
            this.RegisteredProgram = registeredProgram;
        }

        /// <summary>
        /// Gets the registered program.
        /// </summary>
        public RegisteredProgram RegisteredProgram { get; private set; }

        /// <summary>
        /// Gets or sets the name which has to be spoken to start the program.
        /// </summary>
        public string Name
        {
            get
            {
                return this.RegisteredProgram.Name;
            }

            set
            {
                this.RegisteredProgram.Name = value;
                this.OnPropertyChanged("Name");
            }
        }

        /// <summary>
        /// Gets or sets the path to the executable.
        /// </summary>
        public string Path
        {
            get
            {
                return this.RegisteredProgram.Path;
            }

            set
            {
                this.RegisteredProgram.Path = value;
                this.OnPropertyChanged("Path");
            }
        }

        #region IDataErrorInfo Members

        /// <summary>
        /// Gets an error message indicating what is wrong with this object.
        /// </summary>
        /// <value></value>
        /// <returns>An error message indicating what is wrong with this object. The default is an empty string ("").</returns>
        public string Error
        {
            get { return this.RegisteredProgram.Error; }
        }

        /// <summary>
        /// Gets the <see cref="System.String"/> with the specified property name.
        /// </summary>
        public string this[string propertyName]
        {
            get { return this.RegisteredProgram[propertyName]; }
        }

        #endregion

        #region Validation

        /// <summary>
        /// Gets a value indicating whether this instance is valid.
        /// </summary>
        /// <value><c>true</c> if this instance is valid; otherwise, <c>false</c>.</value>
        public bool IsValid
        {
            get
            {
                return this.RegisteredProgram.IsValid;
            }
        }

        #endregion Validation
    }
}
